/**
 ** Icons! Screensaver
 ** Copyright (c) 2002, Flaming Sword Productions
 ** Released into the Public Domain
 **
 ** Written by Brian C. Ledbetter <brian@shadowcom.net>
 ** http://www.shadowcom.net/Software/kss/icons/
 **
 ** $Id: Icon.cpp,v 1.1.1.1 2002/02/27 14:43:13 brian Exp $
 **/
#include	<kapp.h>
#include	<kdebug.h>
#include	<qtimer.h>
#include	<qvbox.h>
#include	<qpainter.h>
#include	<qpixmap.h>
#include	<qimage.h>
#include	<qbitmap.h>
#include	<kscreensaver.h>
#include	<kiconloader.h>
#include	"Icon.h"

//#define		MAX_TRY_SIZE		64
#define		MAX_TRY_SIZE		128

/**
 ** Icon::Icon(): Constructor.  If we've been told to load an icon, then do it.
 **/
		Icon::Icon( QWidget* peedee ) :QWidget(peedee), _paintDevice (peedee), _visible (false) { 

	connect( &_timer, SIGNAL(timeout()), this, SLOT(movement()) );

	determineGeometries();
	setSize( QSize( 16, 16 ) );
	setPosition( QPoint( 0, 0 ) );

	QTimer::singleShot( 1000, this, SLOT(initialize()));
}

/**
 ** Icon::~Icon(): Destructor.  I don't really think there's anything to do here.
 **/
		Icon::~Icon() { }

/**
 ** Icon::initialize(): This will be run 1000ms after the icon is created.  It triggers
 ** the entire timing/display system.
 **/
void		Icon::initialize() {

	determineGeometries();
	emit refresh( this );
}

/**
 ** Icon::setIcon(): Someone has told us to change our icon.  Let's do it.
 **/
void		Icon::setIcon( const QString& icon ) {

	emit loadIcon( icon );
}

/**
 ** Icon::show(): We've been told to show ourselves.  Randomize our starting position,
 ** set our initial size, and start the timer.
 **/
void		Icon::show() {

	// TRAP: Can't show an active icon.  Somebody is confused.
	if( _timer.isActive() ) return;

	_visible = true;
	randomPosition();
	randomVector();

	_visibleSize = 0;
	emit paintEvent( 0L );
	_timer.start( 30 );
}

/**
 ** Icon::hide(): We've either hit the end of our loop, or we've been told to hide 
 ** ourselves.
 **/
void		Icon::hide() {

	_visible = false;
	_timer.stop();

	emit refresh( this );
	emit nextIcon();
}

/**
 ** Icon::randomPosition(): Set a random initial position.  (Initially, this won't be
 ** random at all.)
 **/
void		Icon::randomPosition() {

	QPoint p( 0, (_maxPosition.y() / 2) );

	_startingPosition = p;
	setPosition( p );
}

/**
 ** Icon::randomVector(): Set a random initial vector (Direction, Velocity).
 **/
void		Icon::randomVector() {

	setVector( QPoint( 0, 4.5 ) );
}

/**
 ** Icon::setSize(): Set an arbitrary size.  We will need to redraw our icon from the
 ** closest size (initially, just from the largest icon in-memory...)
 **/
void		Icon::setSize( const QSize& sz ) {

	_size = sz;
	QWidget::resize( sz );
}

/**
 ** Icon::setPosition(): Set an arbitrary position on the screen.
 **/
void		Icon::setPosition( const QPoint& pos ) {

	_position = pos;
	QWidget::move( pos );
}

/**
 ** Icon::setVector(): Set an arbitrary VECTOR.
 **/
void		Icon::setVector( const QPoint& vector ) {

	_vector = vector;
}

/**
 ** Icon::movement(): This is our movement event.  It is called until such time as
 ** we are unable to move any longer, at which case, it hides.
 **/
void		Icon::movement() {

	if(!_visible) return;

	emit clearEvent();

	double Vx, Vy;

	Vx = cos( _vector.x() ) * _vector.y();
	Vy = sin( _vector.x() ) * _vector.y();

	QPoint p = _position;
	p.setX( p.x() + Vx );

	if( (p.x() < (_maxPosition.x() / 2)) && (_visibleSize<_icons.count()-1)) {
		_visibleSize++;
	} else if( (p.x() > (_maxPosition.x() / 2)) && (_visibleSize>0)) {
		_visibleSize--;
	}

	float offset;
	if( _icons[_visibleSize].height() != _smallestSize ) {
		offset = ((_icons[_visibleSize].height()/2) - (_smallestSize/2))/2;
	} else offset = 0;

	p.setY( _startingPosition.y() - offset );
	setPosition( p );

	emit paintEvent( 0L );

	if( p.x() > _maxPosition.x() ) {
		emit hide();
	}
}

/**
 ** Icon::loadIcon(): Handle the loading of all icons that we can find by this name.
 ** Make a note of the biggest and smallest icons that we find.
 **/
void		Icon::loadIcon( const QString& icon ) {

	_smallestSize = _largestSize = 16;
	_icons.clear();

	for( int i = _smallestSize; i <= MAX_TRY_SIZE; i++ ) {
		QPixmap ix = DesktopIcon( icon, i );
		if( !ix.isNull() ) {
			_icons.append( ix );

			if( ix.width() > ix.height() ) {
				_largestSize = ix.width();
			} else {
				_largestSize = ix.width();
			}
		}
	}
}

/**
 ** Icon::determineGeometries(): Find out what the geometry of our display
 ** is.
 **/
void		Icon::determineGeometries() {

	if(!_paintDevice) return;
	_maxPosition = QPoint( _paintDevice->width(), _paintDevice->height() );
}

/**
 ** Icon::paintEvent(): Override our QWidget::paintEvent() method
 ** with one which will only paint to the screensaver window.  (We
 ** do this because of the rediculous way in which KScreenSavers write
 ** to the screen...)
 **/
void            Icon::paintEvent( QPaintEvent *qpe ) {

	QPainter p( _paintDevice );
	p.drawPixmap( _position, _icons[_visibleSize] );
}

/**
 ** Icon::clearEvent(): Time to undraw our icon.  This process stinks.
 **/
void		Icon::clearEvent() {

	QPainter p( _paintDevice );

	QPixmap px( _icons[_visibleSize].width(), _icons[_visibleSize].height() );
	px.fill( Qt::black );

	p.drawPixmap( _position, px );	
}
