/**
 ** Icons! Screensaver
 ** Copyright (c) 2002, Flaming Sword Productions
 ** Released into the Public Domain
 **
 ** Written by Brian C. Ledbetter <brian@shadowcom.net>
 ** http://www.shadowcom.net/Software/kss/icons/
 **
 ** $Id: IconMgr.cpp,v 1.1.1.1 2002/02/27 14:43:13 brian Exp $
 **/
#include	<kstddirs.h>
#include	<kapp.h>
#include	<kdebug.h>
#include	<qdir.h>
#include	"IconMgr.h"
#include	"Icon.h"

#define		MAX_ICONS		3
#define		MAX_VISIBLE		MAX_ICONS-2

/**
 ** IconMgr::IconMgr(): CONSTRUCTOR.  Prepare ourselves.
 **/
		IconMgr::IconMgr( QWidget *peedee )
		:QWidget( peedee ), _paintDevice( peedee ), _currentIcon (0) {

	buildIconList();
	for( int i = 0; i < MAX_ICONS; i++ ) {
		Icon *ic = new Icon( _paintDevice );
		connect(ic, SIGNAL(refresh(Icon*)), this, SLOT(loadIcon(Icon*)));
		connect(ic, SIGNAL(nextIcon()), this, SLOT(showIcon()));
		_icons.append( ic );
	}
	QTimer::singleShot( 1000, this, SLOT(showIcon()));
}

/**
 ** IconMgr::~IconMgr(): DESTRUCTOR.  Clean up.
 **/
		IconMgr::~IconMgr() {

	_icons.clear();
}

/**
 ** IconMgr::loadIcon(): The icon in question has instructed us
 ** that it wishes to be populated with a new image.  Do so.
 **/
void		IconMgr::loadIcon( Icon *ic ) {

	if( !ic ) return;

	QString nic = getNewIcon();
	if( nic.isNull() ) return;

	ic->setIcon( nic );
}

/**
 ** IconMgr::showIcon(): Show another icon, if we can.
 **/
void		IconMgr::showIcon() {

	int x = 0;
	Icon *nextIcon;

	++_currentIcon;
	if( _currentIcon >= _icons.count() ) _currentIcon = 0;

	QListIterator<Icon> it( _icons );
	while( it.current() ) {
		if( x++ == _currentIcon ) {
			_showingIcon = true;
			emit (it.current())->show();
			break;
		}
		++it;
	}
}

/**
 ** IconMgr::buildIconList(): For all icons which we're able to find
 ** in the 48x48 directory, add its symbolic name to our dictionary.
 ** It is from this list which we will select random pictures to put
 ** in begging Icons.  (THIS SHOULD ONLY BE CALLED AT STARTUP!)
 **/
void		IconMgr::buildIconList() {

	/**
	 ** DEFAULT CASE: Find 48x48/apps...
	 **/
	QString base_path = locate( "icon", "hicolor/48x48/apps/go.png" );
	kdDebug() << "Base Path? " << base_path << endl;

	/**
	 ** FALLBACK 1: If that can't be found, try looking for 32x32/apps...
	 **/
	if( base_path.isNull() ) {
		QString base_path = locate( "icon", "hicolor/32x32/apps/go.png" );
		kdDebug() << "Retry Base Path? " << base_path << endl;
	}

	/**
	 ** FALLBACK 2: And if not that, find locolor/32x32/apps...
	 **/
	if( base_path.isNull() ) {
		QString base_path = locate( "icon", "locolor/32x32/apps/go.png" );
		kdDebug() << "Retry Base Path? " << base_path << endl;
	}

	/**
	 ** FINAL FALLBACK: Default to the FreeBSD location for this path.  This is for
	 ** my box only, and was used exclusively in the _very_ early development of this
	 ** app.
	 **/
	if( base_path.isNull() ) base_path = "/usr/local/share/icons/hicolor/48x48/apps";

	/**
	 ** Do some munging to pull out the _REAL_ base_path from our full path...
	 **/
	if( base_path.find( "/apps" ) > 0 ) {
		base_path = base_path.left( base_path.find( "/apps" ) );
	}

	kdDebug() << "BASE PATH: " << base_path << endl;

	processIconDir( base_path );
}

/**
 ** IconDir::processIconDir(): This is a recursive function to help me get everything
 ** loaded properly.  Enter it by pointing us at the root of the icons tree.
 **/
void		IconMgr::processIconDir( const QString& path ) {

	QDir dir( path );

	// TRAP- Path DNE, return.
	if( !dir.exists() || !dir.count() ) return;

	kdDebug() << "IconMgr::processIconDir(): " << path << endl;

	for( unsigned int i = 0; i < dir.count(); i++ ) {
		if( dir[i][0] == '.' ) continue; // SKIP /^\..*/
		QString file = dir[i];
		if( file.find( ".png" )>0 ) {
			file = file.left( file.find(".png") );
			_availableIconIndex.append( file );
		} else if( QDir( path + '/' + dir[i] ).exists() ) {
			processIconDir( path + '/' + dir[i] );
		}
	}
}

/**
 ** IconMgr::getNewIcon(): Select an icon name from our dictionary
 ** at random.
 **/
QString		IconMgr::getNewIcon() {

	int rand, max = _availableIconIndex.count()-1;
	while( (rand = kapp->random()) > max ) ;
	return _availableIconIndex[ rand ];
}
